@@ -171,10 +171,20 @@ def test_open_glob(dir_server):
 
 
 @pytest.mark.network
-@pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
+@pytest.mark.parametrize(
+    "engine",
+    [
+        "pyarrow",
+        pytest.param(
+            "fastparquet", marks=pytest.mark.filterwarnings("ignore::FutureWarning")
+        ),
+    ],
+)
 def test_parquet(engine):
     pytest.importorskip("requests", minversion="2.21.0")
     dd = pytest.importorskip("dask.dataframe")
+    if dd._dask_expr_enabled() and engine == "fastparquet":
+        pytest.skip("fastparquet not supported with dask-expr")
     pytest.importorskip(engine)
     df = dd.read_parquet(
         [
