@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import contextlib
 import itertools
 import multiprocessing as mp
 import os
@@ -46,6 +47,8 @@ try:
 except ImportError:
     pa = None
 
+DASK_EXPR_ENABLED = dd._dask_expr_enabled()
+
 dsk = {
     ("x", 0): pd.DataFrame({"a": [1, 2, 3], "b": [1, 4, 7]}, index=[0, 1, 3]),
     ("x", 1): pd.DataFrame({"a": [4, 5, 6], "b": [2, 5, 8]}, index=[5, 6, 8]),
@@ -54,26 +57,46 @@ dsk = {
 meta = make_meta(
     {"a": "i8", "b": "i8"}, index=pd.Index([], "i8"), parent_meta=pd.DataFrame()
 )
-d = dd.DataFrame(dsk, "x", meta, [0, 4, 9, 9])
+if DASK_EXPR_ENABLED:
+    d = dd.repartition(pd.concat(dsk.values()), [0, 4, 9, 9])
+    shuffle_func = lambda df, *args, **kwargs: df.shuffle(*args, **kwargs)
+    shuffle = lambda df, *args, **kwargs: df.shuffle(*args, **kwargs)
+else:
+    d = dd.DataFrame(dsk, "x", meta, [0, 4, 9, 9])
+    shuffle_func = (
+        shuffle  # type: ignore[assignment]  # conflicts with keyword argument
+    )
 full = d.compute()
 
-shuffle_func = shuffle  # conflicts with keyword argument
-
 
 def test_shuffle(shuffle_method):
-    s = shuffle_func(d, d.b, shuffle=shuffle_method)
+    s = shuffle_func(d, d.b, shuffle_method=shuffle_method)
     assert isinstance(s, dd.DataFrame)
     assert s.npartitions == d.npartitions
 
-    x = dask.get(s.dask, (s._name, 0))
-    y = dask.get(s.dask, (s._name, 1))
+    if not DASK_EXPR_ENABLED:
+        x = dask.get(s.dask, (s._name, 0))
+        y = dask.get(s.dask, (s._name, 1))
 
-    assert not (set(x.b) & set(y.b))  # disjoint
+        assert not (set(x.b) & set(y.b))  # disjoint
     assert set(s.dask).issuperset(d.dask)
 
     assert shuffle_func(d, d.b)._name == shuffle_func(d, d.b)._name
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="no deprecation necessary")
+def test_shuffle_deprecated_shuffle_keyword(shuffle_method):
+    from dask.dataframe.tests.test_multi import list_eq
+
+    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
+        result = dd.shuffle.shuffle(d, d.b, shuffle=shuffle_method)
+    list_eq(result, d)
+
+    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
+        result = d.shuffle(d.b, shuffle=shuffle_method)
+    list_eq(result, d)
+
+
 def test_default_partitions():
     assert shuffle(d, d.b).npartitions == d.npartitions
 
@@ -81,7 +104,7 @@ def test_default_partitions():
 def test_shuffle_npartitions(shuffle_method):
     df = pd.DataFrame({"x": np.random.random(100)})
     ddf = dd.from_pandas(df, npartitions=10)
-    s = shuffle(ddf, ddf.x, shuffle=shuffle_method, npartitions=17, max_branch=4)
+    s = shuffle(ddf, ddf.x, shuffle_method=shuffle_method, npartitions=17, max_branch=4)
     sc = s.compute()
     assert s.npartitions == 17
     assert set(s.dask).issuperset(set(ddf.dask))
@@ -94,7 +117,7 @@ def test_shuffle_npartitions(shuffle_method):
 def test_shuffle_npartitions_lt_input_partitions(shuffle_method):
     df = pd.DataFrame({"x": np.random.random(100)})
     ddf = dd.from_pandas(df, npartitions=20)
-    s = shuffle(ddf, ddf.x, shuffle=shuffle_method, npartitions=5, max_branch=2)
+    s = shuffle(ddf, ddf.x, shuffle_method=shuffle_method, npartitions=5, max_branch=2)
     sc = s.compute()
     assert s.npartitions == 5
     assert set(s.dask).issuperset(set(ddf.dask))
@@ -108,14 +131,15 @@ def test_index_with_non_series(shuffle_method):
     from dask.dataframe.tests.test_multi import list_eq
 
     list_eq(
-        shuffle(d, d.b, shuffle=shuffle_method), shuffle(d, "b", shuffle=shuffle_method)
+        shuffle(d, d.b, shuffle_method=shuffle_method),
+        shuffle(d, "b", shuffle_method=shuffle_method),
     )
 
 
 def test_index_with_dataframe(shuffle_method):
-    res1 = shuffle(d, d[["b"]], shuffle=shuffle_method).compute()
-    res2 = shuffle(d, ["b"], shuffle=shuffle_method).compute()
-    res3 = shuffle(d, "b", shuffle=shuffle_method).compute()
+    res1 = shuffle(d, d[["b"]], shuffle_method=shuffle_method).compute()
+    res2 = shuffle(d, ["b"], shuffle_method=shuffle_method).compute()
+    res3 = shuffle(d, "b", shuffle_method=shuffle_method).compute()
 
     assert sorted(res1.values.tolist()) == sorted(res2.values.tolist())
     assert sorted(res1.values.tolist()) == sorted(res3.values.tolist())
@@ -126,14 +150,14 @@ def test_shuffle_from_one_partition_to_one_other(shuffle_method):
     a = dd.from_pandas(df, 1)
 
     for i in [1, 2]:
-        b = shuffle(a, "x", npartitions=i, shuffle=shuffle_method)
+        b = shuffle(a, "x", npartitions=i, shuffle_method=shuffle_method)
         assert len(a.compute(scheduler="sync")) == len(b.compute(scheduler="sync"))
 
 
 def test_shuffle_empty_partitions(shuffle_method):
     df = pd.DataFrame({"x": [1, 2, 3] * 10})
     ddf = dd.from_pandas(df, npartitions=3)
-    s = shuffle(ddf, ddf.x, npartitions=6, shuffle=shuffle_method)
+    s = shuffle(ddf, ddf.x, npartitions=6, shuffle_method=shuffle_method)
     parts = compute_as_if_collection(dd.DataFrame, s.dask, s.__dask_keys__())
     for p in parts:
         assert s.columns == p.columns
@@ -208,14 +232,22 @@ def test_set_index_general(npartitions, shuffle_method):
 
     ddf = dd.from_pandas(df, npartitions=npartitions)
 
-    assert_eq(df.set_index("x"), ddf.set_index("x", shuffle=shuffle_method))
-    assert_eq(df.set_index("y"), ddf.set_index("y", shuffle=shuffle_method))
-    assert_eq(df.set_index("z"), ddf.set_index("z", shuffle=shuffle_method))
-    assert_eq(df.set_index(df.x), ddf.set_index(ddf.x, shuffle=shuffle_method))
+    assert_eq(df.set_index("x"), ddf.set_index("x", shuffle_method=shuffle_method))
+    assert_eq(df.set_index("y"), ddf.set_index("y", shuffle_method=shuffle_method))
+    assert_eq(df.set_index("z"), ddf.set_index("z", shuffle_method=shuffle_method))
+    assert_eq(df.set_index(df.x), ddf.set_index(ddf.x, shuffle_method=shuffle_method))
     assert_eq(
-        df.set_index(df.x + df.y), ddf.set_index(ddf.x + ddf.y, shuffle=shuffle_method)
+        df.set_index(df.x + df.y),
+        ddf.set_index(ddf.x + ddf.y, shuffle_method=shuffle_method),
     )
-    assert_eq(df.set_index(df.x + 1), ddf.set_index(ddf.x + 1, shuffle=shuffle_method))
+    assert_eq(
+        df.set_index(df.x + 1), ddf.set_index(ddf.x + 1, shuffle_method=shuffle_method)
+    )
+
+    if DASK_EXPR_ENABLED:
+        return
+    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
+        assert_eq(df.set_index("x"), ddf.set_index("x", shuffle=shuffle_method))
 
 
 @pytest.mark.skipif(
@@ -237,7 +269,7 @@ def test_set_index_string(shuffle_method, string_dtype):
     )
     df = df.astype({"y": string_dtype})
     ddf = dd.from_pandas(df, npartitions=10)
-    assert_eq(df.set_index("y"), ddf.set_index("y", shuffle=shuffle_method))
+    assert_eq(df.set_index("y"), ddf.set_index("y", shuffle_method=shuffle_method))
 
 
 def test_set_index_self_index(shuffle_method):
@@ -247,8 +279,13 @@ def test_set_index_self_index(shuffle_method):
     )
 
     a = dd.from_pandas(df, npartitions=4)
-    with pytest.warns(UserWarning, match="this is a no-op"):
-        b = a.set_index(a.index, shuffle=shuffle_method)
+    if DASK_EXPR_ENABLED:
+        ctx = contextlib.nullcontext()
+    else:
+        ctx = pytest.warns(UserWarning, match="this is a no-op")
+
+    with ctx:
+        b = a.set_index(a.index, shuffle_method=shuffle_method)
     assert a is b
 
     assert_eq(b, df.set_index(df.index))
@@ -265,18 +302,18 @@ def test_set_index_names(shuffle_method):
 
     ddf = dd.from_pandas(df, npartitions=4)
 
-    assert set(ddf.set_index("x", shuffle=shuffle_method).dask) == set(
-        ddf.set_index("x", shuffle=shuffle_method).dask
+    assert set(ddf.set_index("x", shuffle_method=shuffle_method).dask) == set(
+        ddf.set_index("x", shuffle_method=shuffle_method).dask
     )
-    assert set(ddf.set_index("x", shuffle=shuffle_method).dask) != set(
-        ddf.set_index("y", shuffle=shuffle_method).dask
-    )
-    assert set(ddf.set_index("x", max_branch=4, shuffle=shuffle_method).dask) != set(
-        ddf.set_index("x", max_branch=3, shuffle=shuffle_method).dask
-    )
-    assert set(ddf.set_index("x", drop=True, shuffle=shuffle_method).dask) != set(
-        ddf.set_index("x", drop=False, shuffle=shuffle_method).dask
+    assert set(ddf.set_index("x", shuffle_method=shuffle_method).dask) != set(
+        ddf.set_index("y", shuffle_method=shuffle_method).dask
     )
+    assert set(
+        ddf.set_index("x", max_branch=4, shuffle_method=shuffle_method).dask
+    ) != set(ddf.set_index("x", max_branch=3, shuffle_method=shuffle_method).dask)
+    assert set(
+        ddf.set_index("x", drop=True, shuffle_method=shuffle_method).dask
+    ) != set(ddf.set_index("x", drop=False, shuffle_method=shuffle_method).dask)
 
 
 ME = "ME" if PANDAS_GE_220 else "M"
@@ -292,7 +329,7 @@ def test_set_index_2(shuffle_method):
         seed=1,
     )
 
-    df2 = df.set_index("name", shuffle=shuffle_method)
+    df2 = df.set_index("name", shuffle_method=shuffle_method)
     df2.value.sum().compute(scheduler="sync")
 
 
@@ -301,7 +338,7 @@ def test_set_index_3(shuffle_method):
     ddf = dd.from_pandas(df, npartitions=5)
 
     ddf2 = ddf.set_index(
-        "x", shuffle=shuffle_method, max_branch=2, npartitions=ddf.npartitions
+        "x", shuffle_method=shuffle_method, max_branch=2, npartitions=ddf.npartitions
     )
     df2 = df.set_index("x")
     assert_eq(df2, ddf2)
@@ -341,11 +378,12 @@ def test_shuffle_sort(shuffle_method):
     ddf = dd.from_pandas(df, npartitions=3)
 
     df2 = df.set_index("x").sort_index()
-    ddf2 = ddf.set_index("x", shuffle=shuffle_method)
+    ddf2 = ddf.set_index("x", shuffle_method=shuffle_method)
 
-    assert_eq(ddf2.loc[2:3], df2.loc[2:3])
+    assert_eq(ddf2, df2, sort_results=False)
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
 @pytest.mark.parametrize("scheduler", ["threads", "processes"])
 def test_rearrange(shuffle_method, scheduler):
     df = pd.DataFrame({"x": np.random.random(10)})
@@ -353,7 +391,7 @@ def test_rearrange(shuffle_method, scheduler):
     ddf2 = ddf.assign(_partitions=ddf.x % 4)
 
     result = rearrange_by_column(
-        ddf2, "_partitions", max_branch=32, shuffle=shuffle_method
+        ddf2, "_partitions", max_branch=32, shuffle_method=shuffle_method
     )
     assert result.npartitions == ddf.npartitions
     assert set(ddf.dask).issubset(result.dask)
@@ -367,6 +405,7 @@ def test_rearrange(shuffle_method, scheduler):
         assert sum(i in set(part._partitions) for part in parts) == 1
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
 def test_rearrange_cleanup():
     df = pd.DataFrame({"x": np.random.random(10)})
     ddf = dd.from_pandas(df, npartitions=4)
@@ -375,7 +414,9 @@ def test_rearrange_cleanup():
     tmpdir = tempfile.mkdtemp()
 
     with dask.config.set(temporay_directory=str(tmpdir)):
-        result = rearrange_by_column(ddf2, "_partitions", max_branch=32, shuffle="disk")
+        result = rearrange_by_column(
+            ddf2, "_partitions", max_branch=32, shuffle_method="disk"
+        )
         result.compute(scheduler="processes")
 
     assert len(os.listdir(tmpdir)) == 0
@@ -385,6 +426,7 @@ def mock_shuffle_group_3(df, col, npartitions, p):
     raise ValueError("Mock exception!")
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
 def test_rearrange_disk_cleanup_with_exception():
     # ensure temporary files are cleaned up when there's an internal exception.
 
@@ -398,13 +440,14 @@ def test_rearrange_disk_cleanup_with_exception():
         with dask.config.set(temporay_directory=str(tmpdir)):
             with pytest.raises(ValueError, match="Mock exception!"):
                 result = rearrange_by_column(
-                    ddf2, "_partitions", max_branch=32, shuffle="disk"
+                    ddf2, "_partitions", max_branch=32, shuffle_method="disk"
                 )
                 result.compute(scheduler="processes")
 
     assert len(os.listdir(tmpdir)) == 0
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
 def test_rearrange_by_column_with_narrow_divisions():
     from dask.dataframe.tests.test_multi import list_eq
 
@@ -478,17 +521,26 @@ def test_set_index_divisions_2():
     assert list(result.compute(scheduler="sync").index[-2:]) == ["d", "d"]
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
 def test_set_index_divisions_compute():
-    d2 = d.set_index("b", divisions=[0, 2, 9], compute=False)
-    d3 = d.set_index("b", divisions=[0, 2, 9], compute=True)
+    deprecated = pytest.warns(
+        FutureWarning, match="the 'compute' keyword is deprecated"
+    )
+
+    with deprecated:
+        d2 = d.set_index("b", divisions=[0, 2, 9], compute=False)
+    with deprecated:
+        d3 = d.set_index("b", divisions=[0, 2, 9], compute=True)
 
     assert_eq(d2, d3)
     assert_eq(d2, full.set_index("b"))
     assert_eq(d3, full.set_index("b"))
     assert len(d2.dask) > len(d3.dask)
 
-    d4 = d.set_index(d.b, divisions=[0, 2, 9], compute=False)
-    d5 = d.set_index(d.b, divisions=[0, 2, 9], compute=True)
+    with deprecated:
+        d4 = d.set_index(d.b, divisions=[0, 2, 9], compute=False)
+    with deprecated:
+        d5 = d.set_index(d.b, divisions=[0, 2, 9], compute=True)
     exp = full.copy()
     exp.index = exp.b
     assert_eq(d4, d5)
@@ -497,6 +549,7 @@ def test_set_index_divisions_compute():
     assert len(d4.dask) > len(d5.dask)
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="constructor doesn't work")
 def test_set_index_divisions_sorted():
     p1 = pd.DataFrame({"x": [10, 11, 12], "y": ["a", "a", "a"]})
     p2 = pd.DataFrame({"x": [13, 14, 15], "y": ["b", "b", "c"]})
@@ -548,34 +601,21 @@ def _set_index(i, df, idx):
     return df.set_index(idx).divisions
 
 
-def test_set_index_reduces_partitions_small(shuffle_method):
-    df = pd.DataFrame({"x": np.random.random(100)})
-    ddf = dd.from_pandas(df, npartitions=50)
-
-    ddf2 = ddf.set_index("x", shuffle=shuffle_method, npartitions="auto")
-    assert ddf2.npartitions < 10
-
-
 def make_part(n):
     return pd.DataFrame({"x": np.random.random(n), "y": np.random.random(n)})
 
 
-def test_set_index_reduces_partitions_large(shuffle_method):
-    nbytes = 1e6
-    nparts = 50
-    n = int(nbytes / (nparts * 8))
-    ddf = dd.DataFrame(
-        {("x", i): (make_part, n) for i in range(nparts)},
-        "x",
-        make_part(1),
-        [None] * (nparts + 1),
-    )
-    ddf2 = ddf.set_index(
-        "x", shuffle=shuffle_method, npartitions="auto", partition_size=nbytes
-    )
-    assert 1 < ddf2.npartitions < 20
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="auto not supported")
+def test_npartitions_auto_raises_deprecation_warning():
+    df = pd.DataFrame({"x": range(100), "y": range(100)})
+    ddf = dd.from_pandas(df, npartitions=10, name="x", sort=False)
+    with pytest.warns(FutureWarning, match="npartitions='auto'"):
+        ddf.set_index("x", npartitions="auto")
+    with pytest.warns(FutureWarning, match="npartitions='auto'"):
+        ddf.sort_values(by=["x"], npartitions="auto")
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="auto not supported")
 def test_set_index_doesnt_increase_partitions(shuffle_method):
     nparts = 2
     nbytes = 1e6
@@ -586,17 +626,18 @@ def test_set_index_doesnt_increase_partitions(shuffle_method):
         make_part(1),
         [None] * (nparts + 1),
     )
-    ddf2 = ddf.set_index(
-        "x", shuffle=shuffle_method, npartitions="auto", partition_size=nbytes
-    )
+    ddf2 = ddf.set_index("x", shuffle_method=shuffle_method, partition_size=nbytes)
     assert ddf2.npartitions <= ddf.npartitions
 
 
 def test_set_index_detects_sorted_data(shuffle_method):
     df = pd.DataFrame({"x": range(100), "y": range(100)})
-    ddf = dd.from_pandas(df, npartitions=10, name="x", sort=False)
+    if DASK_EXPR_ENABLED:
+        ddf = dd.from_pandas(df, npartitions=10, sort=False)
+    else:
+        ddf = dd.from_pandas(df, npartitions=10, name="x", sort=False)
 
-    ddf2 = ddf.set_index("x", shuffle=shuffle_method)
+    ddf2 = ddf.set_index("x", shuffle_method=shuffle_method)
     assert len(ddf2.dask) < ddf.npartitions * 4
 
 
@@ -686,17 +727,18 @@ def test_set_index(engine):
         # NOTE: engine == "cudf" requires cudf/dask_cudf,
         # will be skipped by non-GPU CI.
 
-        dask_cudf = pytest.importorskip("dask_cudf")
+        pytest.importorskip("dask_cudf")
 
     dsk = {
         ("x", 0): pd.DataFrame({"a": [1, 2, 3], "b": [4, 2, 6]}, index=[0, 1, 3]),
         ("x", 1): pd.DataFrame({"a": [4, 5, 6], "b": [3, 5, 8]}, index=[5, 6, 8]),
         ("x", 2): pd.DataFrame({"a": [7, 8, 9], "b": [9, 1, 8]}, index=[9, 9, 9]),
     }
-    d = dd.DataFrame(dsk, "x", meta, [0, 4, 9, 9])
 
-    if engine == "cudf":
-        d = dask_cudf.from_dask_dataframe(d)
+    if DASK_EXPR_ENABLED:
+        d = dd.repartition(pd.concat(dsk.values()), [0, 4, 9, 9])
+    else:
+        d = dd.DataFrame(dsk, "x", meta, [0, 4, 9, 9])
 
     full = d.compute()
 
@@ -719,6 +761,22 @@ def test_set_index(engine):
     assert_eq(d5, full.set_index(["b"]))
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not deprecated")
+def test_set_index_deprecated_shuffle_keyword(shuffle_method):
+    df = pd.DataFrame({"x": [4, 1, 1, 3, 3], "y": [1.0, 1, 1, 1, 2]})
+    ddf = dd.from_pandas(df, 2)
+
+    expected = df.set_index("x")
+
+    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
+        result = ddf.set_index("x", shuffle=shuffle_method)
+    assert_eq(result, expected)
+
+    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
+        result = dd.shuffle.set_index(ddf, "x", shuffle=shuffle_method)
+    assert_eq(result, expected)
+
+
 @pytest.mark.parametrize(
     "engine", ["pandas", pytest.param("cudf", marks=pytest.mark.gpu)]
 )
@@ -772,6 +830,9 @@ def test_set_index_interpolate_int(engine):
     assert all(np.issubdtype(type(x), np.integer) for x in d1.divisions)
 
 
+@pytest.mark.skipif(
+    DASK_EXPR_ENABLED, reason="we don't do division inference for 1 partition frames"
+)
 @pytest.mark.parametrize(
     "engine", ["pandas", pytest.param("cudf", marks=pytest.mark.gpu)]
 )
@@ -799,6 +860,9 @@ def test_set_index_interpolate_large_uint(engine):
     assert set(d1.divisions) == {612509347682975743, 616762138058293247}
 
 
+@pytest.mark.skipif(
+    DASK_EXPR_ENABLED, reason="we don't do division inference for 1 partition frames"
+)
 def test_set_index_timezone():
     s_naive = pd.Series(pd.date_range("20130101", periods=3))
     s_aware = pd.Series(pd.date_range("20130101", periods=3, tz="US/Eastern"))
@@ -905,7 +969,7 @@ def test_set_index_raises_error_on_bad_input():
 
 def test_set_index_sorted_true():
     df = pd.DataFrame({"x": [1, 2, 3, 4], "y": [10, 20, 20, 40], "z": [4, 3, 2, 1]})
-    a = dd.from_pandas(df, 2, sort=False)
+    a = dd.from_pandas(df, 2, sort=False).clear_divisions()
     assert not a.known_divisions
 
     b = a.set_index("x", sorted=True)
@@ -922,8 +986,10 @@ def test_set_index_sorted_true():
             df.set_index(df.x + 1, drop=drop),
         )
 
-    with pytest.raises(ValueError):
-        a.set_index(a.z, sorted=True)
+    if not DASK_EXPR_ENABLED:
+        # we don't validate this
+        with pytest.raises(ValueError):
+            a.set_index(a.z, sorted=True)
 
 
 def test_set_index_sorted_single_partition():
@@ -983,10 +1049,12 @@ def test_set_index_on_empty(converter):
     assert actual.npartitions == 1
     assert all(pd.isnull(d) for d in actual.divisions)
 
-    actual = ddf[ddf.y > df.y.max()].set_index("x", sorted=True)
-    assert assert_eq(actual, expected, check_freq=False)
-    assert actual.npartitions == 1
-    assert all(pd.isnull(d) for d in actual.divisions)
+    if not DASK_EXPR_ENABLED:
+        # we don't recompute divisions in dask-expr
+        actual = ddf[ddf.y > df.y.max()].set_index("x", sorted=True)
+        assert assert_eq(actual, expected, check_freq=False)
+        assert actual.npartitions == 1
+        assert all(pd.isnull(d) for d in actual.divisions)
 
 
 def test_set_index_categorical():
@@ -1029,10 +1097,13 @@ def test_compute_divisions():
         {"x": [1, 2, 3, 4], "y": [10, 20, 20, 40], "z": [4, 3, 2, 1]},
         index=[1, 3, 10, 20],
     )
-    a = dd.from_pandas(df, 2, sort=False)
+    a = dd.from_pandas(df, 2, sort=False).clear_divisions()
     assert not a.known_divisions
 
-    b = compute_and_set_divisions(copy(a))
+    if DASK_EXPR_ENABLED:
+        b = a.compute_current_divisions(set_divisions=True)
+    else:
+        b = compute_and_set_divisions(copy(a))
 
     assert_eq(a, b, check_divisions=False)
     assert b.known_divisions
@@ -1071,8 +1142,8 @@ def test_gh_2730():
     tm.assert_frame_equal(result.sort_values("KEY").reset_index(drop=True), expected)
 
 
-@pytest.mark.parametrize("npartitions", [None, "auto"])
-def test_set_index_does_not_repeat_work_due_to_optimizations(npartitions):
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="we test this over in dask-expr")
+def test_set_index_does_not_repeat_work_due_to_optimizations():
     # Atomic counter
     count = itertools.count()
 
@@ -1090,11 +1161,12 @@ def test_set_index_does_not_repeat_work_due_to_optimizations(npartitions):
     dsk.update({("x", i): (make_part, ("inc", i), n) for i in range(nparts)})
     ddf = dd.DataFrame(dsk, "x", make_part(None, 1), [None] * (nparts + 1))
 
-    ddf.set_index("x", npartitions=npartitions)
+    ddf.set_index("x")
     ntimes = next(count)
     assert ntimes == nparts
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="we don't support inplace")
 def test_set_index_errors_with_inplace_kwarg():
     df = pd.DataFrame({"a": [9, 8, 7], "b": [6, 5, 4], "c": [3, 2, 1]})
     ddf = dd.from_pandas(df, npartitions=1)
@@ -1140,20 +1212,19 @@ def test_disk_shuffle_with_compression_option(compression):
         test_shuffle("disk")
 
 
-@pytest.mark.parametrize("compression", ["UNKOWN_COMPRESSION_ALGO"])
-def test_disk_shuffle_with_unknown_compression(compression):
+def test_disk_shuffle_with_unknown_compression():
     # test if dask raises an error in case of fault config string
-    with dask.config.set({"dataframe.shuffle.compression": compression}):
+    with dask.config.set({"dataframe.shuffle.compression": "UNKOWN_COMPRESSION_ALGO"}):
         with pytest.raises(
             ImportError,
             match=(
                 "Not able to import and load {} as compression algorithm."
                 "Please check if the library is installed and supported by Partd.".format(
-                    compression
+                    "UNKOWN_COMPRESSION_ALGO"
                 )
             ),
         ):
-            test_shuffle("disk")
+            shuffle_func(d, d.b, shuffle_method="disk").compute()
 
 
 def test_disk_shuffle_check_actual_compression():
@@ -1201,9 +1272,14 @@ def test_dataframe_shuffle_on_arg(on, ignore_index, max_branch, shuffle_method):
     else:
         ext_on = df_in[on].copy()
     df_out_1 = df_in.shuffle(
-        on, shuffle=shuffle_method, ignore_index=ignore_index, max_branch=max_branch
+        on,
+        shuffle_method=shuffle_method,
+        ignore_index=ignore_index,
+        max_branch=max_branch,
+    )
+    df_out_2 = df_in.shuffle(
+        ext_on, shuffle_method=shuffle_method, ignore_index=ignore_index
     )
-    df_out_2 = df_in.shuffle(ext_on, shuffle=shuffle_method, ignore_index=ignore_index)
 
     assert_eq(df_out_1, df_out_2, check_index=(not ignore_index))
 
@@ -1230,7 +1306,9 @@ def test_set_index_overlap_2():
         )
     )
     ddf = dd.from_pandas(df, npartitions=2)
-    result = ddf.reset_index().repartition(8).set_index("index", sorted=True)
+    result = (
+        ddf.reset_index().repartition(npartitions=8).set_index("index", sorted=True)
+    )
     expected = df.reset_index().set_index("index")
     assert_eq(result, expected)
     assert result.npartitions == 8
@@ -1252,14 +1330,23 @@ def test_compute_current_divisions_nan_partition():
     a = d[d.a > 3].sort_values("a")
     divisions = a.compute_current_divisions("a")
     assert divisions == (4, 5, 8, 9)
-    a.divisions = divisions
+
+    if DASK_EXPR_ENABLED:
+        # We don't support this
+        pass
+    else:
+        a.divisions = divisions
     assert_eq(a, a, check_divisions=False)
 
     # Compute divisions with 0 null partitions
     a = d[d.a > 1].sort_values("a")
     divisions = a.compute_current_divisions("a")
     assert divisions == (2, 4, 7, 9)
-    a.divisions = divisions
+    if DASK_EXPR_ENABLED:
+        # We don't support this
+        pass
+    else:
+        a.divisions = divisions
     assert_eq(a, a, check_divisions=False)
 
 
@@ -1281,11 +1368,12 @@ def test_compute_current_divisions_overlap_2():
         )
     )
     ddf1 = dd.from_pandas(data, npartitions=2)
-    ddf2 = ddf1.clear_divisions().repartition(8)
+    ddf2 = ddf1.clear_divisions().repartition(npartitions=8)
     with pytest.warns(UserWarning, match="Partitions have overlapping values"):
         ddf2.compute_current_divisions()
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not valid for dask-expr")
 def test_shuffle_hlg_layer():
     # This test checks that the `ShuffleLayer` HLG Layer
     # is used (as expected) for a multi-stage shuffle.
@@ -1293,7 +1381,7 @@ def test_shuffle_hlg_layer():
         pd.DataFrame({"a": np.random.randint(0, 10, 100)}), npartitions=10
     )
     # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-    ddf_shuffled = ddf.shuffle("a", max_branch=3, shuffle="tasks")
+    ddf_shuffled = ddf.shuffle("a", max_branch=3, shuffle_method="tasks")
     keys = [(ddf_shuffled._name, i) for i in range(ddf_shuffled.npartitions)]
 
     # Cull the HLG
@@ -1332,15 +1420,17 @@ def test_shuffle_partitions_meta_dtype():
         npartitions=10,
     )
     # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-    ddf_shuffled = ddf.shuffle(ddf["a"] % 10, max_branch=3, shuffle="tasks")
+    ddf_shuffled = ddf.shuffle(ddf["a"] % 10, max_branch=3, shuffle_method="tasks")
     # Cull the HLG
-    dsk = ddf_shuffled.__dask_graph__()
+    if not DASK_EXPR_ENABLED:
+        dsk = ddf_shuffled.__dask_graph__()
 
-    for layer in dsk.layers.values():
-        if isinstance(layer, dd.shuffle.ShuffleLayer):
-            assert layer.meta_input["_partitions"].dtype == np.int64
+        for layer in dsk.layers.values():
+            if isinstance(layer, dd.shuffle.ShuffleLayer):
+                assert layer.meta_input["_partitions"].dtype == np.int64
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not valid for dask-expr")
 @pytest.mark.parametrize(
     "npartitions",
     [
@@ -1353,7 +1443,7 @@ def test_shuffle_hlg_layer_serialize(npartitions):
         pd.DataFrame({"a": np.random.randint(0, 10, 100)}), npartitions=npartitions
     )
     # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-    ddf_shuffled = ddf.shuffle("a", max_branch=3, shuffle="tasks")
+    ddf_shuffled = ddf.shuffle("a", max_branch=3, shuffle_method="tasks")
 
     # Ensure shuffle layers can be serialized and don't result in
     # the underlying low-level graph being materialized
@@ -1432,13 +1522,14 @@ def test_set_index_with_series_uses_fastpath():
     assert_eq(res, expected)
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not valid for dask-expr")
 def test_set_index_partitions_meta_dtype():
     ddf = dd.from_pandas(
         pd.DataFrame({"a": np.random.randint(0, 10, 100)}, index=np.random.random(100)),
         npartitions=10,
     )
     # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-    ddf = ddf.set_index("a", shuffle="tasks")
+    ddf = ddf.set_index("a", shuffle_method="tasks")
     # Cull the HLG
     dsk = ddf.__dask_graph__()
 
@@ -1447,6 +1538,7 @@ def test_set_index_partitions_meta_dtype():
             assert layer.meta_input["_partitions"].dtype == np.int64
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not valid for dask-expr")
 def test_sort_values_partitions_meta_dtype_with_divisions():
     with dask.config.set({"dataframe.shuffle.method": "tasks"}):
         ddf = dd.from_pandas(
@@ -1460,7 +1552,7 @@ def test_sort_values_partitions_meta_dtype_with_divisions():
             npartitions=10,
         )
         # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-        ddf = ddf.set_index("a", shuffle="tasks").sort_values("b")
+        ddf = ddf.set_index("a", shuffle_method="tasks").sort_values("b")
         # Cull the HLG
         dsk = ddf.__dask_graph__()
 
@@ -1486,6 +1578,20 @@ def test_sort_values(nelem, by, ascending):
     dd.assert_eq(got, expect, check_index=False, sort_results=False)
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not deprecated")
+def test_sort_values_deprecated_shuffle_keyword(shuffle_method):
+    np.random.seed(0)
+    df = pd.DataFrame()
+    df["a"] = np.ascontiguousarray(np.arange(10)[::-1])
+    df["b"] = np.arange(100, 10 + 100)
+    ddf = dd.from_pandas(df, npartitions=10)
+
+    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
+        got = ddf.sort_values(by=["a"], shuffle=shuffle_method)
+    expect = df.sort_values(by=["a"])
+    dd.assert_eq(got, expect, check_index=False, sort_results=False)
+
+
 @pytest.mark.parametrize(
     "backend", ["pandas", pytest.param("cudf", marks=pytest.mark.gpu)]
 )
@@ -1497,10 +1603,24 @@ def test_sort_values_tasks_backend(backend, by, ascending):
     pdf = pd.DataFrame(
         {"x": range(10), "y": [1, 2, 3, 4, 5] * 2, "z": ["cat", "dog"] * 5}
     )
-    ddf = dd.from_pandas(pdf, npartitions=10).to_backend(backend)
+    ddf = dd.from_pandas(pdf, npartitions=10)
+    if backend == "cudf":
+        ddf = ddf.to_backend(backend)
 
     expect = pdf.sort_values(by=by, ascending=ascending)
-    got = dd.DataFrame.sort_values(ddf, by=by, ascending=ascending, shuffle="tasks")
+    got = dd.DataFrame.sort_values(
+        ddf, by=by, ascending=ascending, shuffle_method="tasks"
+    )
+    dd.assert_eq(got, expect, sort_results=False)
+
+    if DASK_EXPR_ENABLED:
+        return
+    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
+        got = dd.DataFrame.sort_values(ddf, by=by, ascending=ascending, shuffle="tasks")
+    dd.assert_eq(got, expect, sort_results=False)
+
+    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
+        got = ddf.sort_values(by=by, ascending=ascending, shuffle="tasks")
     dd.assert_eq(got, expect, sort_results=False)
 
 
@@ -1562,7 +1682,7 @@ def test_shuffle_by_as_list():
     df = pd.DataFrame({"a": [1, 3, 2]})
     ddf = dd.from_pandas(df, npartitions=3)
     with dask.config.set(scheduler="single-threaded"):
-        got = ddf.sort_values(by=["a"], npartitions="auto", ascending=True)
+        got = ddf.sort_values(by=["a"], ascending=True)
     expect = pd.DataFrame({"a": [1, 2, 3]})
     dd.assert_eq(got, expect, check_index=False)
 
@@ -1597,8 +1717,18 @@ def test_sort_values_bool_ascending():
     ddf = dd.from_pandas(df, npartitions=10)
 
     # attempt to sort with list of ascending booleans
-    with pytest.raises(NotImplementedError):
+    with pytest.raises(ValueError, match="length"):
         ddf.sort_values(by="a", ascending=[True, False])
+    with pytest.raises(ValueError, match="length"):
+        ddf.sort_values(by=["a", "b"], ascending=[True])
+    assert_eq(
+        ddf.sort_values(by="a", ascending=[True]),
+        df.sort_values(by="a", ascending=[True]),
+    )
+    assert_eq(
+        ddf.sort_values(by=["a", "b"], ascending=[True, False]),
+        df.sort_values(by=["a", "b"], ascending=[True, False]),
+    )
 
 
 @pytest.mark.parametrize("npartitions", [1, 3])
@@ -1706,12 +1836,15 @@ def test_shuffle_nulls_introduced():
     ddf1 = dd.from_pandas(df1, npartitions=10)
     ddf2 = dd.from_pandas(df2, npartitions=1)
     meta = pd.Series(dtype=int, index=pd.Index([], dtype=bool, name="A"), name="A")
+    include_groups = {"include_groups": False} if PANDAS_GE_220 else {}
     result = (
         dd.merge(ddf1, ddf2, how="outer", on="B")
         .groupby("A")
-        .apply(lambda df: len(df), meta=meta)
+        .apply(lambda df: len(df), meta=meta, **include_groups)
     )
     expected = (
-        pd.merge(df1, df2, how="outer", on="B").groupby("A").apply(lambda df: len(df))
+        pd.merge(df1, df2, how="outer", on="B")
+        .groupby("A")
+        .apply(lambda df: len(df), **include_groups)
     )
     assert_eq(result, expected, check_names=False)
